""" Code generation for explicitly specified tensors """

from dune.perftool.generation import (get_counted_variable,
                                      kernel_cached,
                                      instruction,
                                      temporary_variable,
                                      )

import pymbolic.primitives as prim
import numpy as np


def define_list_tensor(name, expr, visitor, stack=()):
    for i, child in enumerate(expr.ufl_operands):
        from ufl.classes import ListTensor
        if isinstance(child, ListTensor):
            define_list_tensor(name, child, visitor, stack=stack + (i,))
        else:
            instruction(assignee=prim.Subscript(prim.Variable(name), stack + (i,)),
                        expression=visitor.call(child),
                        forced_iname_deps=frozenset(visitor.interface.quadrature_inames()),
                        tags=frozenset({"quad"}),
                        )


@kernel_cached
def pymbolic_list_tensor(expr, visitor):
    name = get_counted_variable("listtensor")
    temporary_variable(name,
                       shape=expr.ufl_shape,
                       dtype=np.float64,
                       managed=True,
                       )
    define_list_tensor(name, expr, visitor)
    return prim.Variable(name)
